#!/usr/bin/python
import moveit_commander
import rospy
import sys
import cv2
import numpy as np

from planning.baxter import Baxter
from planning.scene import Scene
import model as pred

def run(baxter, learner, scene=None):
    rospy.sleep(1)
    inp = None
    while inp != 'q':
        print('p: predict, q: quit')
        if scene is not None:
            print('[opt] write a color:')
        inp = raw_input()
        if scene is not None and inp != 'p' and inp != 'q':
            scene.set_item(inp.lower())

        baxter.left.move(baxter.left.top)
        rospy.sleep(1)
        img = baxter.left.take_snapshot()
        # cv2.imwrite('test.jpg', img)
        p = predict(learner, img)
        print(p)

def predict(learner, img):
    # cv2.imwrite('test.jpg', img)
    img = pred.preprocess_sample(img)
    return pred.predict_features(learner, img)

if __name__ == '__main__':
    moveit_commander.roscpp_initialize(sys.argv)
    rospy.init_node('predict_images', anonymous=True)

    b = Baxter()

    scene = None
    model_path = '.'
    if len(sys.argv) > 1:
        model_path = sys.argv[1]
    if len(sys.argv) > 2:
        scene = Scene(b.get_planning_frame())
        rospy.sleep(2)
        scene.clear_scene()
        scene.set_table()
        scene.set_item()
        rospy.sleep(2)

    b.both.move(b.both.neutral)
    rospy.sleep(1)
    b.left.take_snapshot()
    learner = pred.load_model(model_path, [None, 100, 110, 3], 3)
    run(b, learner, scene)
