#!/usr/bin/python
import moveit_commander
import rospy
import sys
import cv2
import numpy as np
import uuid
import pickle
import os.path
from tqdm import tqdm

from planning.baxter import Baxter
from planning.scene import Scene

def add_samples(x, y, baxter, features, n=50, dmin=-0.02, dmax=0.02, dir_path='./'):
    # f = 1 if features['color'] == 'BLUE' else 0
    shifts = np.random.uniform(dmin, dmax, (n, 3))
    # start pose
    for arm in [baxter.left, baxter.right]:
        arm.move(arm.top)
        rospy.sleep(1)
        pose = arm.current_pose()
        arm.take_snapshot()
        for i in tqdm(range(shifts.shape[0])):
            shift = shifts[i]
            p = np.copy(pose)
            p[:3] += shift
            try:
                arm.move(list(p))
                rospy.sleep(1)
            except Exception as e:
                print(e)
            img = arm.take_snapshot()
            n = dir_path
            for k in sorted(features.keys()):
                n += features[k]
            cv2.imwrite(n+'.{}.{}.jpg'.format(arm.get_name()[:-4], i), img)
            x.append(img)
            y.append(features)
        arm.move(baxter.left.neutral)
    return x, y


def take_images(baxter, scene=None, dir_path='./'):
    x = []
    y = []
    # features = {'shape': 'CUBE', 'size': 'SMALL'}
    inp = None
    while inp != 'q':
        features = {}
        inp = None
        while inp != 'q' and inp != 'd':
            print('name:feature, d: done, q: quit')
            inp = raw_input()
            feature = inp.split(':')
            if len(feature) == 2:
                features[feature[0]] = feature[1].upper()

        if scene is not None:
            scene.set_item(features['color'].lower())
        if inp != 'q':
            x, y = add_samples(x, y, baxter, features, dir_path=dir_path)


if __name__ == '__main__':
    moveit_commander.roscpp_initialize(sys.argv)
    rospy.init_node('garbage_bot', anonymous=True)

    b = Baxter()

    scene = None
    dir_path = './'
    if len(sys.argv) > 1:
        dir_path = sys.argv[1]
    if len(sys.argv) > 2:
        scene = Scene(b.get_planning_frame())
        rospy.sleep(2)
        scene.clear_scene()
        scene.set_table()
        scene.set_item()
        rospy.sleep(2)

    b.both.move(b.both.neutral)
    rospy.sleep(1)

    take_images(b, scene, dir_path=dir_path)
