import rospkg
import rospy
from moveit_commander import PlanningSceneInterface
from geometry_msgs.msg import PoseStamped
from gazebo_msgs.srv import SpawnModel, DeleteModel
import uuid


class Scene(PlanningSceneInterface):

    table_height = 0.8
    item_height = 0.05
    floor=-0.55

    def __init__(self, frame):
        super(Scene, self).__init__()
        self.frame = frame

    def clear_scene(self):
        for name in self.get_known_object_names():
            self.remove_world_object(name)

    def set_table(self):
        pose = PoseStamped()
        pose.header.frame_id = self.frame
        pose.pose.position.x = 1.0
        pose.pose.position.y = 0.0
        pose.pose.position.z = self.floor
        self.add_box('cafe_table', pose, (1., 1., self.table_height))

        pose.pose.position.z = 0.0 # gazebo reference is not the same as moveit
        self.spawn('cafe_table', pose.pose)

    def set_item(self, color='red'):
        pose = PoseStamped()
        pose.header.frame_id = self.frame
        pose.pose.position.x = 0.75
        pose.pose.position.y = 0.0
        pose.pose.position.z = self.floor + self.table_height/2 + self.item_height/2

        name = 'block{}'.format(uuid.uuid4())
        self.add_box(name, pose, (0.04, 0.04, self.item_height))

        pose.pose.position.x = 0.735
        pose.pose.position.y = -0.025
        pose.pose.position.z = self.table_height # gazebo reference is not the same as moveit
        self.spawn('block_' + color, pose.pose, model_name=name, type='urdf')

    def spawn(self, model, pose, model_name=None, type='sdf'):
        if model_name is None:
            model_name = model

        # get model path
        model_path = rospkg.RosPack().get_path('garbage_bot') + '/models/'
        # Load Model SDF
        xml = ''
        with open (model_path + model + '/model-1_4.' + type, 'r') as f:
            xml=f.read().replace('\n', '')
        if type is 'sdf':
            # Spawn Model SDF
            rospy.wait_for_service('/gazebo/spawn_sdf_model')
            try:
                spawn_sdf = rospy.ServiceProxy('/gazebo/spawn_sdf_model', SpawnModel)
                resp_sdf = spawn_sdf(model_name, xml, '/',
                                     pose, self.frame[1:])
            except rospy.ServiceException, e:
                rospy.logerr('Spawn SDF service call failed: {0}'.format(e))
        else:
            # Spawn Block URDF
            rospy.wait_for_service('/gazebo/spawn_urdf_model')
            try:
                spawn_urdf = rospy.ServiceProxy('/gazebo/spawn_urdf_model', SpawnModel)
                resp_urdf = spawn_urdf(model_name, xml, "/",
                                       pose, self.frame[1:])
            except rospy.ServiceException, e:
                rospy.logerr("Spawn URDF service call failed: {0}".format(e))
